==> sigaddons-7.0.21.12.14.tar.gz: sigaddons-7.0.21.12.14/sigaddons/fiscal/wizard/wizard_cancelar_nfe.py <==
# -*- coding: utf-8 -*-

from openerp.osv import fields, osv
from openerp.tools.translate import _
from nfe.nf_e400 import nfe_400
from nfe.nfc_e import nfce as nfc_e
from datetime import datetime


class wizard_cancelar_nfe(osv.TransientModel):
    _name = 'wizard.cancelar.nfe'
    _description = 'Cancelamento NF-e'

    _columns = {
        'name': fields.char('Motivo Cancelamento', size=255, help="Motivo de Cancelamento da NF-e"),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        parametro = self.pool.get('fiscal.nf.capa')._parametros(cr, uid)
        id_s = context.get('registro_id') or context.get('active_id')
        if isinstance(id_s, (tuple, list)):
            id_s = id_s[0]
        brw_nf = self.pool.get('fiscal.nf.capa').browse(cr, uid, id_s, context)
        company_id = brw_nf.company_id.id
        state = brw_nf.state
        tipo = brw_nf.tipo
        if tipo == 'S' or (tipo == 'E' and brw_nf.nf_propria):
            if state != '3' or context.get('cancelamento_manual'):
                self.pool.get('fiscal.nf.capa').fisnfc_funcao_button_cancelar(cr, uid, [id_s],
                                                                              context)
            elif not brw_nf.fisnfc_tipo_servico:
                id_p = self.pool.get('fiscal.parametro').search(cr, uid, [], context=context)
                if id_p:
                    vals_p = self.pool.get('fiscal.parametro').read(
                        cr, uid, id_p[0], context=context)
                else:
                    raise osv.except_osv(_('Atencao'), _('Parametro do Fiscal nao cadastrado.'))

                obj_pessoa = self.pool.get('adm.pessoa')
                obj_pessoa_end = self.pool.get('adm.pessoa.endereco')
                pessoa_id = self.pool.get('res.company').browse(
                    cr, uid, company_id, context=context).rescon_pessoa_id.id
                if not pessoa_id:
                    raise osv.except_osv(_('Atencao'),
                                         _('Nao existe uma pessoa associada a empresa cadastrada.'))

                enderecos = obj_pessoa.read(cr, uid, pessoa_id, ['endereco_ids'],
                                            context=context).get('endereco_ids')
                estado = False
                if enderecos:
                    for x in obj_pessoa_end.browse(cr, uid, enderecos):
                        if x.endereco_tipo_id.name == 'Comercial':
                            estado = x.uf_id.sigla
                            break
                        if x.endereco_tipo_id.name == 'Normal':
                            estado = x.uf_id.sigla
                            continue
                    if not estado:
                        raise osv.except_osv(_('Atencao'),
                                             _('Cadastre um endereco valido para a empresa.'))
                try:
                    motivo = self.remove_accents(vals['name'])
                except UnicodeEncodeError:
                    motivo = self.remove_accents(vals['name'].encode('utf-8'))
                mod_nf_id = brw_nf.modelo_nf_series_id.id
                read_mod = self.pool.get('fiscal.modelo.nf.series').read(
                    cr, uid, mod_nf_id, context=context)
                if len(motivo) < 15:
                    raise osv.except_osv(_('Atencao!'),
                                         _('A justificativa deve ter no minimo 15 caracteres.'))
                value = {}
                # o cancelamento no ERP deve ser feito antes de cancelar na SEFAZ
                self.pool.get('fiscal.nf.capa').fisnfc_funcao_button_cancelar(cr, uid, [id_s],
                                                                              context)
                # Verifica se a nota jah se encontra cancelada
                if brw_nf.fisnfc_consumidor:
                    processo = self.pool.get('fiscal.nf.capa').verifica_status_nfce_sefaz(
                        cr, uid, brw_nf.id, context)
                else:
                    processo, __ = self.pool.get('fiscal.nf.capa').verifica_status_nfe_sefaz(
                        cr, uid, brw_nf.id, context)

                if processo.resposta.cStat.valor != u'101':  # Cancela nota se nao estiver cancelada
                    if brw_nf.fisnfc_consumidor:
                        proc_nfce = nfc_e(
                            parametro.fiscal.certificado_conteudo,  parametro.fiscal.chave_conteudo,
                            brw_nf.modelo_nf_series_id.ambiente, estado,
                            brw_nf.modelo_nf_series_id.tipo_contingencia)
                        proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio
                        processo = proc_nfce.process.cancelar_nfce(
                            brw_nf.nfe_chave_acesso, brw_nf.nfe_protocolo, motivo)
                    else:
                        nfe = nfe_400(certify=parametro.fiscal.certificado_conteudo,
                                      key=parametro.fiscal.chave_conteudo,
                                      environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                                      contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)
                        processo, __ = nfe.cancelar_nota(
                            brw_nf.nfe_chave_acesso, brw_nf.nfe_protocolo, motivo,
                            vals_p['certificado_conteudo'], vals_p['chave_conteudo'],
                            read_mod['ambiente'], estado, read_mod['tipo_contingencia'])
                    if processo.resposta.cStat.valor == u'128':
                        dhrecbto = datetime.strptime(
                            processo.resposta.retEvento.infEvento.dhRegEvento.valor[:-6],
                            '%Y-%m-%dT%H:%M:%S')
                        dhrecbto = self.pool.get('adm.utils').admutl_converte_tz(
                            cr, uid, dhrecbto, 'America/Sao_Paulo', 'UTC', context=context)
                        self.pool.get('fiscal.evento').create(cr, uid, {
                            'fiseve_xml_env': processo.envio.xml,
                            'fiseve_xml_ret': processo.resposta.xml,
                            'fiseve_tipo': '00110111',  # Cancelamento de NF-e
                            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                            'fiseve_status': processo.resposta.retEvento.infEvento.cStat.valor,
                            'fiseve_motivo': processo.resposta.retEvento.infEvento.xMotivo.valor,
                            'fiseve_lote': False,
                            'fiseve_protocolo': processo.resposta.retEvento.infEvento.nProt.valor,
                            'fiseve_nf_id': brw_nf.id,
                        })
                        if processo.resposta.retEvento.infEvento.cStat.valor in (u'135-155-151'):
                            value['nfe_xml_cancelamento'] = processo.processo_cancelamento_nfe.xml
                            if processo.resposta.retEvento.infEvento.Id.valor:
                                value['nfe_protocolo_cancelamento'] = \
                                    processo.resposta.retEvento.infEvento.Id.valor
                            value['nfe_status'] = '%s - %s' % (
                                processo.resposta.retEvento.infEvento.cStat.valor,
                                processo.resposta.retEvento.infEvento.xMotivo.valor)
                        else:
                            raise osv.except_osv(_('Erro!'), _('%s - %s' % (
                                processo.resposta.retEvento.infEvento.cStat.valor,
                                processo.resposta.retEvento.infEvento.xMotivo.valor)))
                    else:
                        if not processo.resposta.cStat.valor:
                            erro = 'Erro interno na SEFAZ. Tente novamente.'
                        else:
                            erro = '%s - %s.' % (processo.resposta.cStat.valor,
                                                 processo.resposta.xMotivo.valor)
                        raise osv.except_osv(_('Erro!'), erro)
                else:  # Nota fiscal jah se encontra cancelada
                    value['nfe_status'] = '%s - %s' % (processo.resposta.cStat.valor,
                                                       processo.resposta.xMotivo.valor)
                self.pool.get('fiscal.nf.capa').write(cr, uid, id_s, value, context)
        if 'cancelamento_manual' in context:
            msg = _('Cancelamento Manual da Nota Fiscal efetuado com sucesso.')
        else:
            msg = _('Nota Fiscal cancelada com sucesso.')
        self.pool.get('fiscal.nf.capa').message_post(cr, uid, id_s, 'NF-e', msg)

        return super(wizard_cancelar_nfe, self).create(cr, uid, vals, context=context)

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('nota_fiscal_consumidor'):
            result = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fiscal', 'view_fiscal_nfce_form')
        else:
            result = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fiscal', 'view_fiscal_nf_capa_form')
        view_id = result and result[1] or False
        res = {'name': 'Nota Fiscal', 'view_type': 'form', 'view_mode': 'form',
               'view_id': [view_id], 'res_model': 'fiscal.nf.capa', 'type': 'ir.actions.act_window',
               'context': context, 'res_id': context.get('registro_id') or context.get('active_id')}

        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

==> sigaddons-7.0.2310.2.tar.gz: sigaddons-7.0.2310.2/sigaddons/fiscal/wizard/wizard_cancelar_nfe.py <==
# -*- coding: utf-8 -*-

from openerp.osv import fields, osv
from openerp.tools.translate import _
from nfe.nf_e400 import nfe_400
from nfe.nfc_e import nfce as nfc_e
from datetime import datetime


class wizard_cancelar_nfe(osv.TransientModel):
    _name = 'wizard.cancelar.nfe'
    _description = 'Cancelamento NF-e'

    _columns = {
        'name': fields.char('Motivo Cancelamento', size=255, help="Motivo de Cancelamento da NF-e"),
    }

    def create(self, cr, uid, vals, context=None):
        context = context and context.copy() or {}
        parametro = self.pool.get('fiscal.nf.capa')._parametros(cr, uid)
        id_s = context.get('registro_id') or context.get('active_id')
        if isinstance(id_s, (tuple, list)):
            id_s = id_s[0]
        brw_nf = self.pool.get('fiscal.nf.capa').browse(cr, uid, id_s, context)
        company_id = brw_nf.company_id.id
        tipo = brw_nf.tipo
        if tipo == 'S' or (tipo == 'E' and brw_nf.nf_propria):
            if context.get('cancelamento_manual') or not brw_nf.nfe_chave_acesso:
                self.pool.get('fiscal.nf.capa').fisnfc_funcao_button_cancelar(cr, uid, [id_s], context)
            elif not brw_nf.fisnfc_tipo_servico:
                obj_pessoa = self.pool.get('adm.pessoa')
                obj_pessoa_end = self.pool.get('adm.pessoa.endereco')
                pessoa_id = self.pool.get('res.company').browse(
                    cr, uid, company_id, context=context).rescon_pessoa_id.id
                if not pessoa_id:
                    raise osv.except_osv(_('Atencao'),
                                         _('Nao existe uma pessoa associada a empresa cadastrada.'))

                enderecos = obj_pessoa.read(cr, uid, pessoa_id, ['endereco_ids'],
                                            context=context).get('endereco_ids')
                estado = False
                if enderecos:
                    for x in obj_pessoa_end.browse(cr, uid, enderecos):
                        if x.endereco_tipo_id.name == 'Comercial':
                            estado = x.uf_id.sigla
                            break
                        if x.endereco_tipo_id.name == 'Normal':
                            estado = x.uf_id.sigla
                            break
                    if not estado:
                        raise osv.except_osv(_('Atencao'),
                                             _('Cadastre um endereco valido para a empresa.'))
                try:
                    motivo = self.remove_accents(vals['name'])
                except UnicodeEncodeError:
                    motivo = self.remove_accents(vals['name'].encode('utf-8'))
                mod_nf_id = brw_nf.modelo_nf_series_id.id
                read_mod = self.pool.get('fiscal.modelo.nf.series').read(cr, uid, mod_nf_id, context=context)
                if len(motivo) < 15:
                    raise osv.except_osv(_('Atencao!'),
                                         _('A justificativa deve ter no minimo 15 caracteres.'))
                value = {}
                # o cancelamento no ERP deve ser feito antes de cancelar na SEFAZ
                self.pool.get('fiscal.nf.capa').fisnfc_funcao_button_cancelar(cr, uid, [id_s],
                                                                              context)
                # Verifica se a nota jah se encontra cancelada
                if brw_nf.fisnfc_consumidor:
                    processo = self.pool.get('fiscal.nf.capa').verifica_status_nfce_sefaz(
                        cr, uid, brw_nf.id, context)
                else:
                    processo, __ = self.pool.get('fiscal.nf.capa').verifica_status_nfe_sefaz(
                        cr, uid, brw_nf.id, context)

                if processo.resposta.cStat.valor != u'101':  # Cancela nota se nao estiver cancelada
                    if brw_nf.fisnfc_consumidor:
                        proc_nfce = nfc_e(parametro.fiscal.certificado_conteudo,
                                          parametro.fiscal.chave_conteudo,
                                          brw_nf.modelo_nf_series_id.ambiente,
                                          estado,
                                          brw_nf.modelo_nf_series_id.tipo_contingencia)
                        proc_nfce.process.tentativas = parametro.fiscal.fispar_nfce_tentativa_envio
                        processo = proc_nfce.process.cancelar_nfce(
                            brw_nf.nfe_chave_acesso, brw_nf.nfe_protocolo, motivo)
                    else:
                        nfe = nfe_400(certify=parametro.fiscal.certificado_conteudo,
                                      key=parametro.fiscal.chave_conteudo,
                                      environment=brw_nf.modelo_nf_series_id.ambiente, state=estado,
                                      contingency=brw_nf.modelo_nf_series_id.tipo_contingencia)
                        processo, __ = nfe.cancelar_nota(
                            brw_nf.nfe_chave_acesso, brw_nf.nfe_protocolo, motivo,
                            parametro.fiscal.certificado_conteudo, parametro.fiscal.chave_conteudo,
                            read_mod['ambiente'], estado, read_mod['tipo_contingencia'])
                    if processo.resposta.cStat.valor == u'128':
                        dhrecbto = datetime.strptime(
                            processo.resposta.retEvento.infEvento.dhRegEvento.valor[:-6],
                            '%Y-%m-%dT%H:%M:%S')
                        dhrecbto = self.pool.get('adm.utils').admutl_converte_tz(
                            cr, uid, dhrecbto, 'America/Sao_Paulo', 'UTC', context=context)
                        self.pool.get('fiscal.evento').create(cr, uid, {
                            'fiseve_xml_env': processo.envio.xml,
                            'fiseve_xml_ret': processo.resposta.xml,
                            'fiseve_tipo': '00110111',  # Cancelamento de NF-e
                            'fiseve_data': dhrecbto or datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                            'fiseve_status': processo.resposta.retEvento.infEvento.cStat.valor,
                            'fiseve_motivo': processo.resposta.retEvento.infEvento.xMotivo.valor,
                            'fiseve_lote': False,
                            'fiseve_protocolo': processo.resposta.retEvento.infEvento.nProt.valor,
                            'fiseve_nf_id': brw_nf.id,
                        })
                        if processo.resposta.retEvento.infEvento.cStat.valor in (u'135-155-151'):
                            value['nfe_xml_cancelamento'] = processo.processo_cancelamento_nfe.xml
                            if processo.resposta.retEvento.infEvento.Id.valor:
                                value['nfe_protocolo_cancelamento'] = \
                                    processo.resposta.retEvento.infEvento.Id.valor
                            value['nfe_status'] = '%s - %s' % (
                                processo.resposta.retEvento.infEvento.cStat.valor,
                                processo.resposta.retEvento.infEvento.xMotivo.valor)
                        else:
                            raise osv.except_osv(_('Erro!'), _('%s - %s' % (
                                processo.resposta.retEvento.infEvento.cStat.valor,
                                processo.resposta.retEvento.infEvento.xMotivo.valor)))
                    else:
                        if not processo.resposta.cStat.valor:
                            erro = 'Erro interno na SEFAZ. Tente novamente.'
                        else:
                            erro = '%s - %s.' % (processo.resposta.cStat.valor,
                                                 processo.resposta.xMotivo.valor)
                        raise osv.except_osv(_('Erro!'), erro)
                else:  # Nota fiscal jah se encontra cancelada
                    value['nfe_status'] = '%s - %s' % (processo.resposta.cStat.valor,
                                                       processo.resposta.xMotivo.valor)
                self.pool.get('fiscal.nf.capa').write(cr, uid, id_s, value, context)
        if 'cancelamento_manual' in context:
            msg = _('Cancelamento Manual da Nota Fiscal efetuado com sucesso.')
        else:
            msg = _('Nota Fiscal cancelada com sucesso.')
        self.pool.get('fiscal.nf.capa').message_post(cr, uid, id_s, 'NF-e', msg)

        return super(wizard_cancelar_nfe, self).create(cr, uid, vals, context=context)

    @staticmethod
    def remove_accents(text):
        from unicodedata import normalize, category
        unicode_string = unicode(text, 'utf-8') if type(text) == str else text
        text = unicode(filter(lambda c: category(c) != 'Mn', normalize('NFKD', unicode_string)))

        return ''.join([ord(x) < 128 and ord(x) != 92 and x or '' for x in text])

    def cancelar(self, cr, uid, ids, context=None):
        context = context and context.copy() or {}
        if context.get('nota_fiscal_consumidor'):
            result = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fiscal', 'view_fiscal_nfce_form')
        else:
            result = self.pool.get('ir.model.data').get_object_reference(
                cr, uid, 'fiscal', 'view_fiscal_nf_capa_form')
        view_id = result and result[1] or False
        res = {'name': 'Nota Fiscal', 'view_type': 'form', 'view_mode': 'form',
               'view_id': [view_id], 'res_model': 'fiscal.nf.capa', 'type': 'ir.actions.act_window',
               'context': context, 'res_id': context.get('registro_id') or context.get('active_id')}

        return res

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
